# -*- coding: utf-8 -*-
'''
https://github.com/rinatz/ctypes_json/blob/master/ctypes_json.py
'''
from ctypes import Array, Structure, Union, _Pointer, _SimpleCData
from json import JSONEncoder


__version__ = '1.0.0'


class CDataJSONEncoder(JSONEncoder):
    def default(self, obj):
        if isinstance(obj, (Array, list)):
            return [self.default(e) for e in obj]

        if isinstance(obj, _Pointer):
            return self.default(obj.contents) if obj else None

        if isinstance(obj, _SimpleCData):
            return self.default(obj.value)

        if isinstance(obj, (bool, int, float, str)):
            return obj

        if obj is None:
            return obj

        if isinstance(obj, (Structure, Union)):
            result = {}
            anonymous = getattr(obj, '_anonymous_', [])

            for key, *_ in getattr(obj, '_fields_', []):
                value = getattr(obj, key)

                # private fields don't encode
                if key.startswith('_'):
                    continue

                if key in anonymous:
                    result.update(self.default(value))
                else:
                    result[key] = self.default(value)

            return result

        return JSONEncoder.default(self, obj)